#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <math.h>
#include "odicon.h"

#define DIM 100

double penalty = 1000;
int    cost[DIM] = { 500, 70, 93, 25, 500, 30, 20, 10, 10, 5 };
double ramda[DIM] = { 0.45, 0.55, 0.65, 0.75, 0.85, 0.4, 0.3, 0.4, 0.3, 0.2 };


static double factorial(int n) {
    int i;
    double ret = 1;
    static double save[1000];

    assert( n < sizeof(save)/sizeof(save[0]) );
    if (save[n] == 0) {
        for (i=2; i<=n; i++) ret *= i;
        save[n] = ret;
    }
    return save[n];
}

static double F(int k, double lamdaj) {
    int m;
    double ret = 0;

    for (m=0; m<=k; m++) {
        ret += exp(-lamdaj) * pow(lamdaj,m) / factorial(m);
    }
    return ret;
}

static double func(int dim, int x[]) {
    int j, k;
    double ret=0;

    assert(dim <= DIM);
    for (j=0; j<dim; j++) {
        if (x[j] < 0) return PLUS_INFINITY;
    }

    for (k=0;; k++) {
        double term = 1;
        for(j=0; j<dim; j++) {
            term *= F( x[j]+k, ramda[j] );
        }
        ret += 1-term;
        if(term >  0.999) break;
    }
    ret *= penalty;

    for (j=0; j<dim; j++) {
        ret += cost[j] * x[j];
    }

    /* ivector_print(dim, x);  printf("; func(x)=%g\n", ret); */
    return ret;
}

static void test(int dim) {
    double ans;
    int x[DIM] = {100, 40, 10, 30, 40,10,10,50, 50,50};

    printf("F2(1) = %f\n", F(1, 0.55));
    printf("F2(2) = %f\n", F(2, 0.55));
    printf("F2(3) = %f\n", F(3, 0.55));
    printf("F2(4) = %f\n", F(4, 0.55));

    submodular_type = IFF;
    ans = lgconv_minimize2(dim, func, x);

    /*
    submodular_type = WOLFE;
    ans = lgconv_minimize2(dim, func, x);
    */

    ans = lgconv_minimize(dim, func, x, 0);
}

static int read_param(int dim) {
    if (!fvector_scan(1, &penalty)) return FALSE;
    if (!ivector_scan(dim, cost))   return FALSE;
    if (!fvector_scan(dim, ramda))  return FALSE;
    return TRUE;
}

void demo_procurement(const int dim) {
    int i, x[DIM];

    if (dim > DIM) {
        fprintf(stderr, "Too many dimension (%d).\n", dim);
        return;
    }

    for (i=0; i<dim; i++) x[i] = 0;
    if (!read_param(dim)) {
        fprintf(stderr, "Read error.\n");
        return;
    }

    cpu_time_limit = 5; /* 5sec */
    submodular_type = IFF;
    /*lgconv_minimize2(dim, func, x);*/
    lgconv_minimize(dim, func, x, 0);
}
